# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007-2008 Toni Gundogdu
#
# This file is part of clive.
#
# clive is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# clive is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with clive.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

## The module checking classes

__all__ = ['Modules']

## The module checking class
class Modules:

    ## Constructor
    def __init__(self):
        self._lookup = {'urlgrabber':self._import_urlgrabber,
            'newt':self._import_newt, 'feedparser':self._import_feedparser,
            'pysqlite':self._import_pysqlite, 'pycrypto':self._import_pycrypto}
        self._mods = {}            
        self._wrapper('urlgrabber')
        self._wrapper('newt')
        self._wrapper('feedparser')
        self._wrapper('pysqlite')
        self._wrapper('pycrypto')

    def check(self, modname):
        return self._mods[modname]

    def _wrapper(self, modname):
        try:
            ver = self._lookup[modname]()
            error = None
        except ImportError, err:
            error = err
            ver = '-'
        self._mods[modname] = \
            {'installed':ver!='-', 'version':ver, 'error':error}

    def _import_urlgrabber(self):
        import urlgrabber as u
        return u.__version__

    def _import_newt(self):
        import snack as newt
        return '?' # Available: the module itself does not contain version

    def _import_feedparser(self):
        import feedparser as f
        return f.__version__

    def _import_pysqlite(self):
        import sqlite3 as s
        return s.version

    def _import_pycrypto(self):
        import Crypto as c
        return c.__version__
